{% extends "aa_intel_tool/base.html" %}

{% load i18n %}

{% block page_title %}{{ parser_title|title }} » {% translate "Intel Parser" %}{% endblock %}

{% block aa_intel_tool_header %}
    <header>
        <h1>{{ parser_title|title }}</h1>
    </header>

    {% include "aa_intel_tool/partials/scan/evetime.html" %}
    {% include "aa_intel_tool/partials/scan/buttons.html" %}
{% endblock %}

{% block aa_intel_tool_body %}
    <div class="local-scan-result row" id="aa-intel-chatscan-row">
        {% include "aa_intel_tool/partials/scan/chatlist/alliances.html" %}
        {% include "aa_intel_tool/partials/scan/chatlist/corporations.html" %}
        {% include "aa_intel_tool/partials/scan/chatlist/pilots.html" %}
    </div>
{% endblock %}

{% block extra_css %}
    {% include "bundles/datatables-css-bs5.html" %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-css.html" %}
{% endblock %}

{% block extra_javascript %}
    <script>
        const aaIntelToolJsSettingsOverride = {
            url: {
                getPilotList: '{% url "aa_intel_tool:ajax_get_scan_data" scan_hash scan_data_section.PILOTLIST.value %}',
                getCorporationList: '{% url "aa_intel_tool:ajax_get_scan_data" scan_hash scan_data_section.CORPORATIONLIST.value %}',
                getAllianceList: '{% url "aa_intel_tool:ajax_get_scan_data" scan_hash scan_data_section.ALLIANCELIST.value %}'
            }
        }
    </script>

    {% include "bundles/datatables-js-bs5.html" %}
    {% include "bundles/clipboard-js.html" %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-js.html" %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-chatscan-js.html" %}
{% endblock extra_javascript %}
