{% load sri %}

{% include "aa_intel_tool/bundles/aa-intel-tool-scan-result-common-js.html" %}
{% include "aa_intel_tool/bundles/aa-intel-tool-dscan-highlight-js.html" %}

{% sri_static "aa_intel_tool/javascript/aa-intel-tool-dscan.min.js" %}
