# Generated by Django 4.0.10 on 2023-09-22 21:21

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aa_intel_tool", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scan",
            name="raw_data",
            field=models.TextField(verbose_name="Scan raw data"),
        ),
    ]
