{% load i18n %}

<div class="local-scan-result row" id="aa-intel-fleetcomp-participation-row">
    <div class="clearfix">
        <div class="col-sm-6 col-md-4 col-lg-4 clearfix">
            <header class="aa-intel-section-header my-4">
                <h2>
                    {% translate "Participation details" as section_header %}
                    {{ section_header|title }}
                </h2>
            </header>
        </div>
    </div>

    {% include "aa_intel_tool/partials/scan/chatlist/alliances.html" %}
    {% include "aa_intel_tool/partials/scan/chatlist/corporations.html" %}
    {% include "aa_intel_tool/partials/scan/chatlist/pilots.html" %}
</div>
