@media all {
    .aa-intel-tool .aa-intel-scan-page-buttons ul li {
        display: block;
        float: left;
        width: 150px;
    }

    .aa-intel-tool .aa-intel-scan-page-buttons button {
        position: relative;
        width: 100%;
    }

    .aa-intel-tool .aa-intel-copy-result {
        max-width: 310px;
    }

    .aa-intel-tool .paste-explanation ul {
        list-style-type: none;
    }

    .aa-intel-tool .paste-explanation li > small {
        color: var(--bs-gray);
    }

    .aa-intel-tool td.data-align-right {
        text-align: right;
    }

    .aa-intel-tool .aa-intel-section-header {
        border-bottom: 1px solid var(--bs-gray-200);
    }

    .aa-intel-tool .aa-intel-section-header .aa-intel-section-title {
        font-size: 1.25em;
    }

    .aa-intel-tool .aa-intel-eve-image-wrapper {
        display: inline-block;
        margin-right: 0.5rem;
    }

    .aa-intel-tool .aa-intel-eve-information-wrapper {
        display: inline-block;
        vertical-align: inherit;
    }

    .aa-intel-tool .aa-intel-eve-name-wrapper,
    .aa-intel-tool .aa-intel-additional-information-wrapper {
        display: block;
    }
}

/* Highlighting
------------------------------------------------------------------------------------- */
@media all {
    .aa-intel-tool .aa-intel-highlight td:first-child,
    .aa-intel-tool .aa-intel-highlight-sticky td:first-child {
        border-left: 1px solid var(--bs-red);
        padding-left: 3px;
    }
}

/* Form
------------------------------------------------------------------------------------- */
@media all {
    .aa-intel-tool .aa-intel-form-submitted-info {
        display: none;
    }

    .aa-intel-tool form.is-submitting button[type="submit"] {
        cursor: not-allowed;
        opacity: 0.5;
    }

    .aa-intel-tool form.is-submitting .aa-intel-form-submitted-info {
        display: block;
    }
}

/* Loading Spinner
------------------------------------------------------------------------------------- */
@media all {
    svg.aa-intel-loading-spinner {
        height: 1rem;
        width: 1rem;
    }

    .aa-intel-tool .aa-intel-form-submitted-info svg.aa-intel-loading-spinner {
        height: 2rem;
        width: 2rem;
    }
}

/* Data Tables
------------------------------------------------------------------------------------- */
@media all {
    .aa-intel-tool div.dataTables_wrapper div.dataTables_info {
        padding-top: 8px;
        white-space: initial;
    }

    .aa-intel-tool .dataTables_wrapper table.dataTable {
        width: 100%;
    }

    .aa-intel-tool .dataTable tbody tr:hover,
    .aa-intel-tool .dataTable tbody tr.aa-intel-highlight,
    .aa-intel-tool .dataTable tbody tr.aa-intel-highlight-sticky {
        --bs-table-bg-type: var(--bs-table-hover-bg);
        --bs-table-color-type: var(--bs-table-striped-color);
    }

    .aa-intel-tool .table-aa-intel tbody > tr > td {
        vertical-align: middle;
    }

    .aa-intel-tool .table-no-images > tbody > tr > td {
        padding-bottom: 12px;
        padding-top: 11px;
    }

    .aa-intel-tool .table-sortable > tbody > tr > .table-data-count {
        padding-right: 1rem;
        text-align: right;
    }

    .aa-intel-tool td.text-ellipsis {
        max-width: 50px;
        white-space: nowrap;
    }

    .aa-intel-tool .aa-intel-eve-information-wrapper {
        max-width: calc(100% - 35px);
    }

    .aa-intel-tool .aa-intel-eve-name-wrapper {
        overflow: hidden;
        text-overflow: ellipsis;
    }

    .aa-intel-tool td.fix-eve-image-position > span.alliance-id-1 {
        transform: none;
    }
}
