{% load i18n %}

<div class="aa-intel-result-row aa-intel-dscan-result row d-none" id="aa-intel-dscan-row-interesting-on-grid">
    <div class="clearfix">
        <div class="col-sm-6 col-md-4 col-lg-4 clearfix">
            <header class="aa-intel-section-header my-4">
                <h2>
                    {% translate "Interesting on grid" as section_header %}
                    {{ section_header|title }}
                </h2>
            </header>
        </div>
    </div>

    {% translate "Upwell structures" as dscan_section_header_upwell_structures %}
    {% translate "Deployables" as dscan_section_header_upwell_deployables %}
    {% translate "POS / POS modules" as dscan_section_header_upwell_starbases %}
    {% translate "Miscellaneous" as dscan_section_header_upwell_miscellaneous %}

    {% include "aa_intel_tool/partials/scan/dscan/interesting-on-grid/items.html" with section_header=dscan_section_header_upwell_structures dscan_section="upwell-structures" %}
    {% include "aa_intel_tool/partials/scan/dscan/interesting-on-grid/items.html" with section_header=dscan_section_header_upwell_deployables dscan_section="deployables" %}
    {% include "aa_intel_tool/partials/scan/dscan/interesting-on-grid/items.html" with section_header=dscan_section_header_upwell_starbases dscan_section="starbases" %}
    {% include "aa_intel_tool/partials/scan/dscan/interesting-on-grid/items.html" with section_header=dscan_section_header_upwell_miscellaneous dscan_section="miscellaneous" %}
</div>
