{% load sri %}

{% include "aa_intel_tool/bundles/aa-intel-tool-scan-result-common-js.html" %}
{% include "aa_intel_tool/bundles/aa-intel-tool-fleetcomposition-highlight-js.html" %}

{% sri_static "aa_intel_tool/javascript/aa-intel-tool-fleetcomposition.min.js" %}

{% if app_settings.INTELTOOL_ENABLE_MODULE_CHATSCAN %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-chatscan-highlight-js.html" with common_already_loaded=True %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-chatscan-js.html" with common_already_loaded=True %}
{% endif %}
