{% load django_bootstrap5 %}
{% load i18n %}

<div class="card card-default mb-3">
    <div class="card-body">
        <div class="row">
            <div class="col-lg-4">
                <div class="paste-explanation mb-4">
                    <header class="entry-header">
                        <p class="h3 entry-title">{% translate "What can I paste?" %}</p>
                    </header>

                    <ul class="p-0">
                        {% if app_settings.INTELTOOL_ENABLE_MODULE_CHATSCAN %}
                            <li>
                                {% translate "Chat member list" as item_title %}
                                » {{ item_title|title }}

                                {% if app_settings.INTELTOOL_CHATSCAN_MAX_PILOTS > 0 %}
                                    <small>
                                        ({% blocktranslate count max_pilots=app_settings.INTELTOOL_CHATSCAN_MAX_PILOTS %}Limited to a maximum of {{ max_pilots }} pilot per scan.{% plural %}Limited to a maximum of {{ max_pilots }} pilots per scan.{% endblocktranslate %})
                                    </small>
                                {% endif %}
                            </li>
                        {% endif %}

                        {% if app_settings.INTELTOOL_ENABLE_MODULE_DSCAN %}
                            <li>» {% translate "D-Scan" %} </li>
                        {% endif %}

                        {% if app_settings.INTELTOOL_ENABLE_MODULE_FLEETCOMP %}
                            <li>
                                {% translate "Fleet composition" as item_title %}
                                » {{ item_title|title }}
                            </li>
                        {% endif %}
                    </ul>

                    <p>
                        {% translate "Please keep in mind, parsing large amounts of data can take some time. Be patient, CCP's API is not the fastest to answer …" %}
                    </p>
                </div>
            </div>

            <div class="col-lg-8">
                <form id="new_intel" name="new_intel" method="post" action="/intel/">
                    {% csrf_token %}

                    {% bootstrap_form form %}

                    {% translate "Submit" as button_text %}
                    {% bootstrap_button button_type="submit" content=button_text %}

                    <div class="col-sm-12 aa-intel-form-submitted-info mt-3">
                        <div class="aa-callout aa-callout-info clearfix">
                            <div class="float-end ms-3">
                                {% include "aa_intel_tool/partials/common/loading-spinner.html" %}
                            </div>

                            <p>{% translate "Working on it, please be patient …" %}</p>

                            <ul>
                                <li>{% translate "Detecting the intel type" %}</li>
                                <li>{% translate "Fetching potentially missing information from ESI (this might take a moment)" %}</li>
                                <li>{% translate "Parsing data" %}</li>
                            </ul>
                        </div>
                    </div>
                </form>
            </div>
        </div>
    </div>
</div>
