{% load i18n %}

<div class="aa-intel-result-row aa-intel-dscan-result row d-none" id="aa-intel-dscan-row-system-information">
    <div class="clearfix">
        <div class="col-sm-6 col-md-4 col-lg-4 clearfix">
            <header class="aa-intel-section-header my-4">
                <h2>
                    {% translate "System information" as section_header %}
                    {{ section_header|title }}
                </h2>
            </header>
        </div>
    </div>
</div>
