{% extends "aa_intel_tool/base.html" %}

{% load evelinks %}
{% load i18n %}

{% block aa_intel_tool_body %}
    {% include "aa_intel_tool/partials/index/form.html" %}
    {% include "aa_intel_tool/partials/body/scan-retention-note.html" %}
{% endblock %}

{% block extra_css %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-css.html" %}
{% endblock %}

{% block extra_javascript %}
    {% include "aa_intel_tool/bundles/aa-intel-tool-js.html" %}
{% endblock extra_javascript %}
