{% extends "allianceauth/base-bs5.html" %}

{% load i18n %}
{% load aa_i18n %}

{% block page_title %}
    {% translate "Intel Parser" %}
{% endblock %}

{% block header_nav_brand %}
    {% translate "Intel Parser" %}
{% endblock header_nav_brand %}

{% block content %}
    <div class="aa-intel-tool">
        <div class="aa-intel-tool-header">
            {% block aa_intel_tool_header %}{% endblock %}
        </div>

        <div class="aa-intel-tool-body">
            {% get_datatables_language_static LANGUAGE_CODE as DT_LANG_PATH %}

            {% translate "Permalink successfully copied" as copyToClipboardSuccess %}
            {% translate "Something went wrong. Nothing copied. Maybe your browser does not support this function." as copyToClipboardError %}
            {% translate "Unaffiliated" as scanDataUnaffiliated %}
            {% translate "No Alliance" as scanDataNoAlliance %}
            {% translate "NPC Corp" as scanDataNpcCorp %}

            <script>
                const aaIntelToolJsSettingsDefaults = {
                    language: {
                        django: '{{ LANGUAGE_CODE }}',
                        dataTables: {url: '{{ DT_LANG_PATH }}'}
                    },
                    url: {},
                    translation: {
                        copyToClipboard: {
                            permalink: {
                                text: {
                                    success: '{{ copyToClipboardSuccess|escapejs }}',
                                    error: '{{ copyToClipboardError|escapejs }}'
                                }
                            }
                        },
                        scanData: {
                            unaffiliated: '{{ scanDataUnaffiliated|escapejs }}',
                            noAlliance: '{{ scanDataNoAlliance|escapejs }}',
                            npcCorp: '{{ scanDataNpcCorp|escapejs }}',
                        }
                    }
                }
            </script>

            {% block aa_intel_tool_body %}{% endblock %}
        </div>

        <div class="aa-intel-tool-footer mt-3 pt-3 border-top border-light">
            {% include "aa_intel_tool/partials/footer/app-translation-footer.html" %}
        </div>
    </div>
{% endblock %}
